﻿using System.ComponentModel;
using System.Resources;

namespace C1.Win.FlexGrid
{
    public partial class C1FlexGridSearchPanel : UserControl
    {
        [DefaultValue(true)]
        [Obsolete("HighlightSearchResults is deprecated, please use SearchHighlightMode instead.")]
        public bool HighlightSearchResults { get; set; } = true;

        [DefaultValue(SearchHighlightMode.OnlyFirst)]
        public SearchHighlightMode SearchHighlightMode { get; set; } = SearchHighlightMode.OnlyFirst;

        [DefaultValue(true)]
        public bool FilterSearchResults { get; set; } = true;

        [DefaultValue(true)]
        public bool SearchThroughVisibleCols { get; set; } = true;

        [DefaultValue(true)]
        public bool SearchThroughVisibleRows { get; set; } = true;

        [DefaultValue(1000)]
        public int SearchDelay
        {
            get { return _timer.Interval; }
            set { _timer.Interval = value; }
        }

        [DefaultValue(SearchMode.SearchClick)]
        public SearchMode SearchMode
        {
            get { return _searchMode; }
            set { _searchMode = value; btnSearch.Visible = value == SearchMode.SearchClick; }
        }


        private SearchMode _searchMode = SearchMode.SearchClick;
        private C1FlexGrid? _grid;
        private readonly string _searchDefinition = "<SearchDefinition />";
        private readonly System.Windows.Forms.Timer _timer = new() { Interval = 1000 };


        public C1FlexGridSearchPanel()
        {
            InitializeComponent();

            var resources = new ResourceManager(typeof(C1FlexGridSearchPanel));
            btnSearch.Image = (Bitmap)resources.GetObject("Search");
            btnCancel.Image = (Bitmap)resources.GetObject("Cancel");
        }

        public void SetC1FlexGridSearchPanel(C1FlexGrid c1flexGrid)
        {
            _grid = c1flexGrid;
            comboBox1.TextChanged += ComboBox1_TextChanged;
            _timer.Tick += _timer_Tick;
        }

        private void ComboBox1_TextChanged(object? sender, EventArgs e)
        {
            if (_searchMode == SearchMode.Always && _grid != null)
            {
                _timer.Start();
            }
        }

        private void _timer_Tick(object? sender, EventArgs e)
        {
            if (string.IsNullOrWhiteSpace(comboBox1.Text))
            {
                ClearSearch();
                _timer.Stop();
                return;
            }
            Searchgrid();
            _timer.Stop();
        }

        private void btnSearch_Click(object? sender, EventArgs e)
        {
            if (_grid != null)
                Searchgrid();
        }
        private void btnCancel_Click(object? sender, EventArgs e)
        {
            if (_grid != null)
                ClearSearch();
        }

        private void Searchgrid()
        {
            if (_grid != null && !string.IsNullOrEmpty(comboBox1.Text) && !string.IsNullOrWhiteSpace(comboBox1.Text))
            {
                _grid.ApplySearch(comboBox1.Text, SearchHighlightMode, FilterSearchResults, SearchThroughVisibleCols, SearchThroughVisibleRows);
                
                if (!comboBox1.Items.Contains(comboBox1.Text))
                {
                    comboBox1.Items.Add(comboBox1.Text);
                }
            }
            if (string.IsNullOrWhiteSpace(comboBox1.Text))
            {
                ClearSearch();
            }
        }

        private void ClearSearch()
        {
            if (_grid != null)
            {
                _grid.SearchDefinition = _searchDefinition;
                comboBox1.Text = string.Empty;
            }
        }

        /// <summary> 
        /// Clean up any resources being used.
        /// </summary>
        /// <param name="disposing">true if managed resources should be disposed; otherwise, false.</param>
        protected override void Dispose(bool disposing)
        {
            if (disposing && (components != null))
            {
                components.Dispose();

                _timer.Tick -= _timer_Tick;
                comboBox1.TextChanged -= ComboBox1_TextChanged;
                btnSearch.Click -= btnSearch_Click;
                btnCancel.Click -= btnCancel_Click;
            }
            base.Dispose(disposing);
        }
    }

    public enum SearchMode
    {
        Always,
        SearchClick
    }
}
